/*
 * Book.java
 *
 * Created on 1. Februar 2004, 13:22
 */

package j3d_uebung.geometry;
import com.sun.j3d.utils.universe.*;
import com.sun.j3d.utils.geometry.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import javax.media.j3d.Appearance;
import java.awt.Color;
import j3d_uebung.behavior.PickBehavior;
import com.sun.j3d.utils.image.TextureLoader;

/**
 *
 * @author  arnd
 */
public class Book extends java.awt.Component {

    /** Creates a new instance of Book */
    public Book()  {
    }



    public static Group createBook(Canvas3D canvas) {
        //Dimensionen Front- und Backcover
        //Achtung: Es sind die Ausdehnungen
        //entlang der Achsen gemeint
        //Eine Box, die mit diesen Werten erzeugt wird,
        //hat die doppelte Groesse
        float height = 0.5f;
        float width = 0.4f;
        float thickness = 0.05f;

        //Dicke des Seitenstapels
        float pageStackThickness = 0.2f;

        //Wurzel des Subgrafen
        BranchGroup book = new BranchGroup();


        // Erzeuge Appearance-Objekte fr Cover, Seiten und speziell die erste Buchseite
        Appearance coverAppearance = new Appearance();
        Appearance pagesAppearance = new Appearance();
        Appearance firstPageAppearance = new Appearance();

        // Material fr Cover und Buchseiten setzen
        coverAppearance.setMaterial(new Material(new Color3f(Color.BLACK),new Color3f(Color.BLACK), new Color3f(0.0f,0.0f,1.0f),new Color3f(Color.BLACK),0.5f));
        pagesAppearance.setMaterial(new Material(new Color3f(Color.BLACK),new Color3f(Color.BLACK), new Color3f(1.0f,1.0f,1.0f),new Color3f(Color.BLACK),0.5f));

        //URL fr Textur holen
        java.net.URL url = ClassLoader.getSystemResource("./j3d_uebung/pics/paper.jpg");

        //TexturLoader zum Laden der fr die Textur 1. Buchseite benutzen
        TextureLoader loader = new TextureLoader(url,  canvas);

        //Appearance fr erste Seite mit Textur erzeugen
        firstPageAppearance.setTexture(loader.getTexture());

        //Box fr Frontcover erzeugen
        Box front =  new Box(width, height, thickness, coverAppearance);

        //Box fr Backcover erzeugen
        Box back = new Box(width, height, thickness, coverAppearance);

	//Box fr Seiten erzeugen
        Box pages = new Box(width, height, pageStackThickness, Box.GENERATE_TEXTURE_COORDS | Box.GENERATE_NORMALS, pagesAppearance);

        //Appearance fr erste Seite setzen
        pages.getShape(Box.FRONT).setAppearance(firstPageAppearance);

        //Box fr Buchrcken erzeugen
        Box side = new Box(thickness, height, pageStackThickness+2*thickness, coverAppearance);

  	//Verschiebung nach links und vorn
        Transform3D beforeRot = new Transform3D();
        beforeRot.set(new Vector3f(-width, 0.0f, thickness+pageStackThickness));
        TransformGroup beforeRotTG = new TransformGroup(beforeRot);

        //TransformGroup fr Rotation des Frontcovers anlegen
        TransformGroup frontRotTG = new TransformGroup(new Transform3D());

	//Schreiben und Lesen der TransformGroup fr Rotation erlauben
        frontRotTG.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
        frontRotTG.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
        
        //Translation nach rechts (fr Frontcover)
        Transform3D afterRotTransform = new Transform3D();
        afterRotTransform.set(new Vector3f(width, 0.0f, 0.0f));
        TransformGroup afterRotTG = new TransformGroup(afterRotTransform);

	//Zweig zusammenbauen:
        afterRotTG.addChild(front);
        frontRotTG.addChild(afterRotTG);
        beforeRotTG.addChild(frontRotTG);
        book.addChild(beforeRotTG);

	//Backcover nach hinten verschieben
        Transform3D backTransform = new Transform3D();
        backTransform.set(new Vector3f(0.0f, 0.0f, -(thickness+pageStackThickness)));
        TransformGroup backTG = new TransformGroup(backTransform);
        backTG.addChild(back);
        book.addChild(backTG);

	//Buchrcken nach links verschieben
        Transform3D sideTransform = new Transform3D();
        sideTransform.set(new Vector3f(-(width+thickness), 0.0f, -0.0f));
        TransformGroup sideTG = new TransformGroup(sideTransform);

        //Buchrcken zur Transformgruppe hinzufgen
        sideTG.addChild (side);

        //Transformgruppe fr Buchrcken zum Zweig hinzufgen
        book.addChild(sideTG);

	//Seiten zum Buch hinzufgen
        book.addChild(pages);
        
        //neues PickBehavior erzeugen
        PickBehavior behavior = new PickBehavior(canvas, book, front, frontRotTG);
        
        //Behavior zum Buch hinzufgen
        book.addChild(behavior);
        
        //Buch kompilieren
        book.compile();
        
        
        return book;

    }




}
